/**
 * Toy example: proof that the Michelson system has a symmetric periodic solution
 * 
 * Settings:
 * S = {(x,y,0): x,y\in R} - Poincare section
 * P:S->S - Poincare map
 * Poincare map is reverisble, i.e.
 * R(x,y) = R(-x,y)
 * If P^n(x,y) exists then   
 * R(P^n(x,y)) = P^n(R(x,y))
 * 
 * Methodology:
 * If P^n(0,y0) = (0,y1) for some y0 then (0,y0) is a symmetric periodic point
 *
 * (0,y) - an approximate periodic point
 * r>0 - some constant
 * 1. Check that \pi_x P^n(0,[y-r,y+r]) exists
 * 2. Chck that 
 *          \pi_x P^n(0,y-r) * \pi_x P^n(0,y+r) < 0 
 *    This implies that  
 *          \pi_x P^n(0,y0) = 0
 *  for some y0 \in [y-r,y+r] 
 */ 
 
#include <iostream>
#include "capd/capdlib.h"

using namespace capd;
using namespace std;

double findSymPO(double y, double iteration){
  DMap vf("var:x,y,z;fun:y,z,1-y-0.5*x^2;"); // vector field
  DOdeSolver solver(vf, 20);                 // ODE solver
  DCoordinateSection section(3, 2);          // section z=0
  DPoincareMap pm(solver, section);
  
  // Bisection search - as bad as possible 
  // but we did not introduce derivatives of Poincare map, yet.
  double e=0.2;
  double y1 = y-e, y2=y+e;
  DVector u1({0.,y1,0.});
  DVector u2({0.,y2,0.});
  for(int i=0;i<iteration;++i) u1 = pm(u1);
  for(int i=0;i<iteration;++i) u2 = pm(u2);
  while(y2-y1>1e-14){
    y = (y1+y2)/2;
    DVector u({0.,y,0.});
    for(int i=0;i<iteration;++i) u = pm(u);
    if(u1[0]*u[0]<0){
      u2 = u;
      y2 = y;
    } else {
      u1 = u;
      y1 = y;
    }
  }
  cout << "y=" << y << endl;
  cout << "P(0,y,0) \\approx " << (u1+u2)/2 << endl;  
  return y;
}

void validateSymPO(interval y, int iteration){
  IMap vf("var:x,y,z;fun:y,z,1-y-0.5*x^2;"); // vector field
  IOdeSolver solver(vf, 20);                 // 20th order solver    
  ICoordinateSection section(3, 2);          // section z=0
  IPoincareMap pm(solver, section);

  // Check that the Poincare map is defined on (0,y)
  C0HOTripletonSet s(IVector({0.,y,0.}));
  std::cout << "P(0,y)=" << pm(s,iteration) << std::endl;
  
  // Compute P(0,left(y)) and P(0,right(y))
  C0HOTripletonSet s1(IVector({0.,y.left(),0.}));
  C0HOTripletonSet s2(IVector({0.,y.right(),0.}));
  IVector r1 = pm(s1,iteration);
  IVector r2 = pm(s2,iteration);
  // Check that x-coordinate changes the sign
  std::cout << "check inclusion? " << ( r1[0]*r2[0]<0 ) << std::endl;
}

int main()
{
  cout.precision(17);
  cout << boolalpha;
  try {
    double y1 = findSymPO(1.5,1);
    double y2 = findSymPO(0.5,2);

    validateSymPO(y1 + interval(-1,1)*1e-13, 1);
    validateSymPO(y2 + interval(-1,1)*1e-13, 2);  
  
  } catch (exception& e) {
    cout << "\n\nException caught: " << e.what() << endl;
  }
  return 0;
}

/* Output:
y=1.5259617305036892
P(0,y,0) \approx {2.581268532253489e-15,-2.2911863313523244,2.7755575615628914e-17}
y=0.50002564853520548
P(0,y,0) \approx {3.8857805861880479e-16,-2.4502895657581703,5.5511151231257827e-17}
P(0,y)={[-4.1872942551283562e-13, 4.1641174840269898e-13],[-2.291186331352558, -2.2911863313521015],[-3.9945812071632077e-13, 3.9945812071634854e-13]}
check inclusion? true
P(0,y)={[-4.3770776164184129e-13, 4.2537625715394545e-13],[-2.450289565758581, -2.4502895657577808],[-9.1976675538868059e-13, 9.1976675538878762e-13]}
check inclusion? true

*/
