/////////////////////////////////////////////////////////////////////////////
//  Oscilator 
/// @{
 
/////////////////////////////////////////////////////////////////////////////
/// @file OscillatorConstantForcing.cpp
///
/// Integrates controlled oscilator equation
/// 
///    x' = y
///    y' = -x + e(t)
///       where e(t) \in [- \epsilon, \epsilon]
///
///  This example shows that when we just plug interval [- \epsilon, \epsilon] 
///  into integration then:
///     * e(t) is treated as constant forcing 
///     * it brings overestimations due to dependency problem.
///  To avoid this we treat e as additional variable and add equation e'=0  
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) Tomasz Kapela 2006. 

#include "capd/capdlib.h"
#include "capd/krak/krak.h"
#include "capd/krak/noframe.h"
//#define _low_level_

using namespace capd;
using namespace std;
using namespace capd::krak;

// uncomment the following line if you wish to wait for a key after each step 
#define _WAIT_FOR_KEY_

Frame fr;

// Ranges for graphical window
double minx=-1.5, maxx=1.5, miny=-1.5,  maxy=1.5;

double eps    = 1.0e-1;                             ///<  maximal forcing
int    nSteps = 30;                                 ///<  number of steps in one period 
double step   = Interval::pi().rightBound()/nSteps; ///<  time step for integration
int    order  = 10;                                 ///<  order of Taylor method

int readKey()
{
  int s=NO_KEY;
  UserMove um;
#ifdef _WAIT_FOR_KEY_
  while(NO_KEY==s) 
#endif
  {
    GetUserMove(um);
    s=um.key;
  }
  return s;
}

void waitForKey()
{
#ifndef _WAIT_FOR_KEY_
    rootFrame << " Hit any key ! ";
    waitBt();
#endif
}

void drawAxes(Frame & fr)
{
    fr.setWorldCoord(minx,miny,maxx,maxy);
    fr.line(0.0,miny,0.0,maxy,BLACK);
    fr.line(minx,0.0,maxx,0.0,BLACK);
}

/// Forcing is some constant from given set e
/// All solutions are bounded.
/// Due to overestimations (mainly wrapping effect)
/// we do not see it. Estimations are blowing up.
void OscilatorWithConstantForcing()
{
  rootFrame.Clear();
  rootFrame << " Oscillator with Constant Forcing \n\n";
  rootFrame << " x' = y \n"
            << " y' = -x + e \n\n"
            << "   where e = [" << eps << " , " << eps << "]\n";  
  drawAxes(fr);
  IMap vfield("par:e;var:x,y;fun:y,-x+e;");
  vfield.setParameter("e",Interval(-eps,eps));

  IOdeSolver solver(vfield, order);
  solver.setStep(step);

  C1Rect2Set S(IVector({1.0,0.0}));
  IVector v(2);
  int s=NO_KEY;
  SetFgCol(BLUE);
  waitBt();
  do{
    S.move(solver);
    v = IVector(S);
    fr.boxFill(v[0].leftBound(), v[1].leftBound(),v[0].rightBound(), v[1].rightBound(),BLUE,SOLID_P);
    fr.dot(v[0].leftBound(),v[1].leftBound());
    s=readKey(); 
    fr.boxFill(v[0].leftBound(), v[1].leftBound(),v[0].rightBound(), v[1].rightBound(),RED,SOLID_P);
    
  }while(s!=EscKey);
  
  waitForKey();
}

/// just ODE: oscilator with sine forcing
/// solution slowly goes to infinity
void OscilatorWithNonConstantForcing()
{
  rootFrame.Clear();
  rootFrame << " Oscillator with Non-Constant Forcing\n\n";
  rootFrame << " x' = y \n"
            << " y' = -x+e*sin(t) \n\n "
            << "  where e = " << eps << "\n";
  drawAxes(fr);
  
  IMap vfield("par:e;time:t;var:x,y;fun:y,-x+e*sin(t);");
  vfield.setParameter("e",Interval(eps));

  IOdeSolver solver(vfield, order);
  solver.setStep(step);
  C1Rect2Set S({-1.,0.});
  IVector v;
  int s=NO_KEY;
  double bSize = 0.01;
  do{
    S.move(solver);
    v = IVector(S);
    fr.boxFill(v[0].leftBound()-bSize, v[1].leftBound()-bSize,v[0].rightBound()+bSize, v[1].rightBound()+bSize,RED);
    fr.dot(v[0].leftBound(),v[1].leftBound());
    s=readKey();
  }while(s!=EscKey);
  waitForKey();
}

/// Oscilator with constant forcing but this time 
/// constant is an additiona variable 
/// to avoid wrapping effect.
/// All solutions are circles (different radii and center)
void OscilatorWithConstantForcing2()
{
  rootFrame.Clear();
  rootFrame << " Oscillator with Constant Forcing as an additional variable\n\n";
  rootFrame << " x' = y \n"
            << " y' = -x+e\n"
            << " e' = 0 \n\n"
            << "  where e(0) = [" << - eps <<", " << eps <<  "]\n";
  rootFrame << " \n Black dots - oscillator with sinus forcing \n";
  drawAxes(fr);

  IMap vf1("var:x,y,e;fun:y,-x+e,0;"),
       vf2("par:e;time:t;var:x,y;fun:y,-x+e*sin(t);");

  vf2.setParameter("e",interval(eps));

  IOdeSolver solver1(vf1, order);
  IOdeSolver solver2(vf2, order);
  solver1.setStep(step);
  solver2.setStep(step);

  C1Rect2Set S1({-1.0,0.0,interval(-eps,eps)});
  C1Rect2Set S2({-1.0,0.0});
  IVector v1, v2;
  double bSize = 0.01;
  int s=NO_KEY;
  do
  {
    S1.move(solver1);
    v1 = IVector(S1);
    fr.boxFill(v1[0].leftBound(), v1[1].leftBound(),v1[0].rightBound(), v1[1].rightBound(),GREEN);
    S2.move(solver2);
    v2 = IVector(S2);
    fr.boxFill(v2[0].leftBound()-bSize, v2[1].leftBound()-bSize,v2[0].rightBound()+bSize, v2[1].rightBound()+bSize,BLACK);
    fr.dot(v2[0].leftBound(),v2[1].leftBound());
    s=readKey();
    fr.boxFill(v1[0].leftBound(), v1[1].leftBound(),v1[0].rightBound(), v1[1].rightBound(),RED);
  }while(s!=EscKey);
  waitForKey();
}

int main(int argc, char* argv[])
{
  try{
    openGW(1000,700,WHITE,BLACK);
    setTextSize(16);
    fr = Frame(300,10,990, 690, WHITE, BLACK);
    OscilatorWithConstantForcing();
    OscilatorWithNonConstantForcing();
    OscilatorWithConstantForcing2();
    closeGW();
    return 0;
 }
 catch(std::exception &e)
 {
   cout << e.what();
   closeGW();
   return 1;
 }
}

/// @}
