/**
 * Toy example: proof that van der Pol oscillator has a periodic solution for mu=1
 * 
 * Settings:
 * S = {(x,0): x\in R} - Poincare section
 * P:S->S - Poincare map
 * 
 * Methodology:
 * (x0,0) - an approximate periodic point
 * r - an interval centered at zero
 * Check that \pi_x P(x0+r,0) \subset x0+r
 * 
 * Take r=interval(-1,1)*1e-5 
 */ 
 
#include <iostream>
#include "capd/capdlib.h"

using namespace capd;
using namespace std;

int main()
{
  cout.precision(17);
  cout << boolalpha;
  try {
    // Formula for vector field of the van der Pol oscillator
    std::string _vf = "var:x,y;fun:y,(1-x^2)*y-x;";

    // Instance of vector field, solver, section and Poincare map
    // Prefix D stands for nonrigorous computation in double precision
    DMap dvf(_vf);
    DOdeSolver dSolver(dvf, 20 /*order of Taylor method*/);
    DCoordinateSection dSection(2, 1); // 2 variables, x-index 0, y has index 1
    DPoincareMap dpm(dSolver, dSection, poincare::PlusMinus);
    
    // The orbit is attracting, so iterate Poincare map starting from (1,0)
    // until |P(u)-u| < 1e-14 
    DVector u({1.,0.});
    double err = 1;
    while(err>1e-14){
      DVector v = dpm(u);
      err = (v-u).euclNorm();
      u = v;
      cout << u << endl;
    }
    cout << endl;
    
    // Instance of vector field, solver, section and Poincare map
    // Prefix I stands for rigorous computation in interval arithmetic
    IMap vf(_vf);
    IOdeSolver solver(vf, 20);
    ICoordinateSection section(2, 1);
    IPoincareMap pm(solver, section, poincare::PlusMinus);
    
    // Take a ball centered at approximate periodic point
    IVector x({u[0]+interval(-1,1)*1e-5,0.});
    
    // This is tripleton representation of the set x. 
    // C0 = no derivatives with respect to space
    // HO = Hermite-Obreschkov numerical method 
    C0HOTripletonSet s(x);
    
    // Call routine that computes rigorously Poincare map
    IVector y = pm(s);
    cout << "x = " << x[0] << endl; 
    cout << "y = " << y[0] << endl;
    
    // check inclusion
    cout << "y-x0 = " << y[0]-u[0] << endl;
    cout << "check inclusion? " << subset(y[0],x[0]) << endl;

  } catch (exception& e) {
    cout << "\n\nException caught: " << e.what() << endl;
  }
  return 0;
}

/* Output:
{1.998515487808342,2.7755575615628914e-17}
{2.008610936573811,0}
{2.0086198532024828,0}
{2.0086198608682473,-1.7347234759768071e-18}
{2.0086198608748376,1.7347234759768071e-18}
{2.0086198608748433,1.7347234759768071e-18}

x = [2.0086098608748433, 2.0086298608748434]
y = [2.0086198481018909, 2.0086198740315355]
y-x0 = [-1.2772952473483201e-08, 1.315669218016069e-08]
check inclusion? true
*/
